@@ -7,7 +7,7 @@ from collections import defaultdict
 
 class Carbon(object):
     def __init__(self, host, port):
-        self.data = {}
+        self.data = []
         self.host = host
         self.port = port
 
@@ -31,37 +31,27 @@ class Carbon(object):
 
     def _send(self, data):
         try:
-            for metric, datapoints in data.items():
-                for dp in datapoints:
-                    logging.debug("Sending: %s %s %s" % (metric, dp[1], dp[0]))
-                    self.sock.sendall("%s %s %s\n" % (metric, dp[1], dp[0]))
+            logging.info("Sending %d total metrics to carbon" % len(data))
+
+            for metric in data:
+                logging.debug("Sending %s" % metric)
+            self.sock.sendall("\n".join(data))
         except socket.error:
-            raise
+            logging.error("Error sending to carbon, trying to reconnect.")
+
+            self.sock = self.connect()
+            while len(data):
+                self.data.append(data.pop())
 
     def send(self):
         buf_sz = 500
-        to_send = defaultdict(list)
+        to_send = []
 
-        for metric in self.data.keys():
-            while len(self.data[metric]) > 0:
-                l = len(to_send)
-                if l < buf_sz:
-                    to_send[metric].append(self.data[metric].pop())
-                else:
-                    try:
-                        self._send(to_send)
-                    except socket.error:
-                        logging.error("Error sending to carbon, trying to reconnect.")
-                        self.sock = self.connect()
+        while self.data:
+            if len(to_send) < buf_sz:
+                to_send.append(self.data.pop())
+            else:
+                self._send(to_send)
+                to_send = []
 
-                        for entry in to_send:
-                            self.data[entry[0]].append(entry[1])
-
-        try:
-            self._send(to_send)
-        except socket.error:
-            logging.error("Error sending to carbon, trying to reconnect.")
-            self.sock = self.connect()
-
-        for entry in to_send:
-            self.data[entry[0]].append(entry[1])
+        self._send(to_send)
